/*
 * Copyright © 2019-2022 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of
 * the following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3LLS_CDT_H
#define ATSC3LLS_CDT_H

// See A/360 5.2.2.2

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t textDataOffset;             // The text string
} ATSC3LLSCDT_CertificatesNode;

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t textDataOffset;             // The text string
} ATSC3LLSCDT_NextCertNode;

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t textDataOffset;             // The text string
} ATSC3LLSCDT_CurrentCertNode;

typedef struct
{
    EnXMLParserNodeBase base;

    int64_t nextCertFromMillisecondsUTC;
    int64_t currentCertUntilMillisecondsUTC;
    ATSC3LLSCDT_NextCertNode nextCertNode;
} ATSC3LLSCDT_CertReplacementNode;

#define ATSC3LLSCDTCertReplacementNodeAttribute_NextCertFrom    0x0001
#define ATSC3LLSCDTCertReplacementNodeAttribute_CurrentCertUntil  0x0002
#define ATSC3LLSCDTCertReplacementChildNode_NextCert            0x1000

typedef struct
{
    EnXMLParserNodeBase base;

    int64_t ocspRefreshSeconds;         // Duration converted to seconds

    EnXMLParserDataOffsetArray certificates; // Array of 
                                        // Base 64 coded certifiates

    ATSC3LLSCDT_CurrentCertNode currentCertNode;

    ATSC3LLSCDT_CertReplacementNode certReplacementNode;
} ATSC3LLSCDT_ToBeSignedDataNode;

#define ATSC3LLCDTToBeSignedDataNodeAttribute_OCSPRefresh       0x0001
#define ATSC3LLCDTToBeSignedDataChildNode_Certificates          0x1000
#define ATSC3LLCDTToBeSignedDataChildNode_CurrentCert           0x2000
#define ATSC3LLCDTToBeSignedDataChildNode_CertReplacment        0x4000

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t textDataOffset;           // The text string.
} ATSC3LLSCDT_CMSSignedDataNode;

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t textDataOffset;           // The text string.
} ATSC3LLSCDT_OCSPResponseNode;

typedef struct _ATSC3LLSCDT_CertificationDataNode
{
    EnXMLParserNodeBase base;

    ATSC3LLSCDT_CMSSignedDataNode cmsSignedDataNode;
    EnXMLParserDataOffsetArray ocspResponses; // Array of OCSPResponseDataNodes

    ATSC3LLSCDT_ToBeSignedDataNode toBeSignedDataNode;
    
    EnXMLParserDataBuffer dataBuffer;
} ATSC3LLSCDT_CertificationDataNode;

#define ATSC3LLSCDTChildNode_CMSSignedData                      0x1000
#define ATSC3LLSCDTChildNode_OCSPResponse                       0x2000
#define ATSC3LLSCDTRTChildNode_ToBeSignedData                   0x4000

typedef enum
{
    ATSC3LLSCDTNodeType_Invalid = 0,
    ATSC3LLSCDTNodeType_CertificationData = 1,
    ATSC3LLSCDTNodeType_CMSSignedData = 2,
    ATSC3LLSCDTNodeType_OCSPResponse = 3,
    ATSC3LLSCDTNodeType_ToBeSignedData = 4,
    ATSC3LLSCDTNodeType_CertReplacement = 5,
    ATSC3LLSCDTNodeType_CurrentCert = 6,
    ATSC3LLSCDTNodeType_NextCert = 7,
    ATSC3LLSCDTNodeType_Certificates = 8,
} ATSC3LLSCDTTNodeType;

ENStatus
ATSC3LLSParseCDT(const char *pXml, int32_t sXml, 
    ATSC3LLSCDT_CertificationDataNode *pCertificationDataNode);

ENStatus
ATSC3LLSFreeParsedCDT(
    ATSC3LLSCDT_CertificationDataNode *pCertificationDataNode);

#endif
