/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3LLS_RRT_H
#define ATSC3LLS_RRT_H

// See A/331 6.5


typedef struct
{
    EnXMLParserNodeBase base;

    int32_t stringDataOffset;           // The text string.

    int32_t langDataOffset;             // The language of the string. 
                                        // Default is 'en'.
} ATSC3LLSRRT_TextTypeNode;

#define ATSC3LLSRRTTextTypeNodeAttribute_lang                   0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t ratingValue;               // The rating level value in integer 
                                        // form.
    EnXMLParserDataOffsetArray ratingValueAbbrevs; // Array of 
                                        // ATSC3LLSRRT_TextTypeNode.
                                        // Abbreviated human-readable string 
                                        // describing the rating value.
    EnXMLParserDataOffsetArray ratingValueStrings; // Array of  
                                        // ATSC3LLSRRT_TextTypeNode.
                                        // Human-readable string describing the 
                                        // rating value.
} ATSC3LLSRRT_RatingNode;

#define ATSC3LLSRRTRatingNodeAttribute_ratingValue              0x0001
#define ATSC3LLSRRTRatingChildNode_RatingValueAbbrev            0x0002
#define ATSC3LLSRRTRatingChildNode_RatingValueString            0x0004

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t dimensionLevels;           // The number of levels for content 
                                        // advisory in this dimension.
    int32_t dimensionGraduated;         // Indicates whether the rating 
                                        // dimension is defined on a "graduated 
                                        // scale".
    EnXMLParserDataOffsetArray dimensionTitles; // Array of 
                                        // ATSC3LLSRRT_TextTypeNode.
                                        // Human-readable string  describing 
                                        // the dimension.
    EnXMLParserDataOffsetArray ratings; // Array of ATSC3LLSRRT_RatingNode. 
                                        // Definition of each rating in the 
                                        // Dimension.
} ATSC3LLSRRT_DimensionNode;

#define ATSC3LLSRRTDimensionNodeAttribute_dimensionLevels       0x0001
#define ATSC3LLSRRTDimensionNodeAttribute_dimensionGraduated    0x0002
#define ATSC3LLSRRTDimensionChildNode_DimensionTitle            0x0004
#define ATSC3LLSRRTDimensionChildNode_Rating                    0x0008

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t regionIdentifier;          // Identifies the rating region 
                                        // described.
    EnXMLParserDataOffsetArray regionIdTexts; // Array of 
                                        // ATSC3LLSRRT_TextTypeNode.
                                        // Human-readable string 
                                        // describing the rating region, e.g. 
                                        // "Canada".
    EnXMLParserDataOffsetArray dimensions; // Array of 
                                        // ATSC3LLSRRT_DimensionNode.
                                        // One or more elements, each 
                                        // describing one rating dimension in 
                                        // the rating region.
} ATSC3LLSRRT_RatingRegionTableNode;

#define ATSC3LLSRRTRatingRegionTableNodeAttribute_regionIdentifier 0x0001
#define ATSC3LLSRRTRatingRegionTableChildNode_RegionIdText      0x0002
#define ATSC3LLSRRTRatingRegionTableChildNode_Dimension         0x0004

typedef struct
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray ratingRegionTables; // Array of
                                        // ATSC3LLSRRT_RatingRegionTableNode.
    
    EnXMLParserDataBuffer dataBuffer;
} ATSC3LLSRRT_RatingRegionTablesNode;

#define ATSC3LLRRTNodeAttribute_XMLNS                           0x0001
#define ATSC3LLSRRTChildNode_RatingRegionTable                  0x0002

typedef enum
{
    ATSC3LLSRRTNodeType_Invalid = 0,
    ATSC3LLSRRTNodeType_RatingRegionTables = 1,
    ATSC3LLSRRTNodeType_RatingRegionTable = 2,
    ATSC3LLSRRTNodeType_RegionIdText = 3,
    ATSC3LLSRRTNodeType_Dimension = 4,
    ATSC3LLSRRTNodeType_DimensionTitle = 5,
    ATSC3LLSRRTNodeType_Rating = 6,
    ATSC3LLSRRTNodeType_RatingValueAbbrev = 7,
    ATSC3LLSRRTNodeType_RatingValueString = 8,
    ATSC3LLSRRTNodeType_TextType = 9,
} ATSC3LLSRRTTNodeType;

ENStatus
ATSC3LLSParseRRT(const char *pXml, int32_t sXml, 
    ATSC3LLSRRT_RatingRegionTablesNode *pRatingRegionTablesNode);

ENStatus
ATSC3LLSFreeParsedRRT(
    ATSC3LLSRRT_RatingRegionTablesNode *pRatingRegionTablesNode);

#endif
