/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3LLS_SLT_H
#define ATSC3LLS_SLT_H

#define MAX_OTHER_BSID 64
#define MAX_SERVICES 64

typedef enum
{
    ATSC3LLSSLTSLSProtocol_Reserved = 0,
    ATSC3LLSSLTSLSProtocol_ROUTE = 1,
    ATSC3LLSSLTSLSProtocol_MMTP = 2,
} ATSC3LLSSLTSLSProtocol;

typedef enum 
{
    ATSC3LLSSLTServiceCategory_Reserved = 0,
    ATSC3LLSSLTServiceCategory_LinearAV = 1,
    ATSC3LLSSLTServiceCategory_LinearAudioOnly = 2,
    ATSC3LLSSLTServiceCategory_AppBased = 3,
    ATSC3LLSSLTServiceCategory_ESG = 4,
    ATSC3LLSSLTServiceCategory_EAS = 5,
} ATSC3LLSSLTServiceCategory;

typedef struct
{
    EnXMLParserNodeBase base;
    int32_t urlType;
    int32_t urlDataOffset;
} ATSC3LLSSLTInetUrlNode;

#define ATSC3LLSSLTInetUrlNodeAttribute_urlType                 0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t slsProtocol;       // Protocol used to deliver the service layer
                                // signaling for this service.
                                // 1: ROUTE, 2: MMTP, other values: ATSC 
                                // reserved
    uint32_t slsMajorProtocolVersion; // Major version number of 
                                // protocol used to deliver Service Layer 
                                // Signaling for this service.
    uint32_t slsMinorProtocolVersion; // Minor version number of 
                                // protocol used to deliver Service Layer 
                                // Signaling for this service.
    IPV4Address slsDestinationIpAddress; // A string containing the 
                                // dotted IPv4 destination address of the
                                // packets carrying broadcast SLS data for this
                                // service.
    uint32_t slsDestinationUdpPort; // Port number of the packets
                                // carrying broadcast SLS data for this
                                // service.
    IPV4Address slsSourceIpAddress; // A string containing the dotted-
                                // IPv4 source address of the packets
                                // carrying broadcast SLS data for this
                                // service.
} ATSC3LLSSLT_BroadcastSvcSignalingNode;

#define ATSC3LLSSLTBroadcastSvcSignalingNodeAttribute_slsProtocol       0x0001
#define ATSC3LLSSLTBroadcastSvcSignalingNodeAttribute_slsMajorProtocolVersion      0x0002
#define ATSC3LLSSLTBroadcastSvcSignalingNodeAttribute_slsMinorProtocolVersion      0x0004
#define ATSC3LLSSLTBroadcastSvcSignalingNodeAttribute_slsDestinationIpAddress      0x0008
#define ATSC3LLSSLTBroadcastSvcSignalingNodeAttribute_slsDestinationUdpPort        0x0010
#define ATSC3LLSSLTBroadcastSvcSignalingNodeAttribute_slsSourceIpAddress           0x0020

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t otherBsid;         // Identifier of other Broadcast Stream that
                                // delivers duplicates or portions of this
                                // service.
    uint32_t type;               // Indicates whether the Broadcast Stream
                                // identified by otherBsid delivers a duplicate
                                // or a portion of this service.
} ATSC3LLSSLT_OtherBsidNode;

#define ATSC3LLSSLTOtherBsidNodeAttribute_type                          0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t serviceId;         // Integer number that identifies this Service
                                // within the scope of this Broadcast area.
    int32_t globalServiceIDDataOffset; // A globally unique URI that identifies 
                                // the ATSC 3.0 Service. This attribute is not 
                                // present for the ESG and EAS streams.
    uint32_t sltSvcSeqNum;      // Version of SLT service info for this service
    int32_t protected;          // Indicates whether one or more components
                                // needed for meaningful presentation of this
                                // service are protected (e.g. encrypted).
    uint32_t majorChannelNo;    // Major channel number of the service
    uint32_t minorChannelNo;    // Minor channel number of the service
    uint32_t serviceCategory;   // Service category per A331 Table 6.4
    int32_t shortServiceNameDataOffset; // Short name of the Service
    int32_t hidden;             // Indicates whether the service is intended 
                                // for testing or proprietary use, and is not
                                // to be selected by ordinary TV receivers
    int32_t broadbandAccessRequired; // Indicates whether broadband access is
                                // required for a receiver to make a
                                // meaningful presentation of the service.
    int32_t essential;          // Indicates if the essential portion of the
                                // Service is delivered via this Broadcast 
                                // Stream.

    uint16_t simulcastTSID;     // Identifer of an ATSC 1.0 broadcast stream
                                // carrying the same programming content.
    uint16_t simulcastMajorChannelNo; // Major channel number of the ATSC
                                // 1.0 service carrying the same programming
                                // content.
    uint16_t simulcastMinorChannelNo; // Minor channel number of the ATSC
                                // 1.0 service carrying the same programming
                                // content.

    int32_t svcCapabilitiesDataOffset; // Required capabilities for 
                                // decoding and meaningfully presenting
                                // content of this service.

    ATSC3LLSSLT_BroadcastSvcSignalingNode broadcastSvcSignaling;

    ATSC3LLSSLTInetUrlNode svcInetUrl;  // URL to access Internet signaling 
                                // for this service.

    EnXMLParserDataOffsetArray otherBsids; // Array of 
                                // ATSC3LLSSLTOtherBsidNode
} ATSC3LLSSLT_ServiceNode;

#define ATSC3LLSSLTServiceNodeAttribute_serviceId                       0x0001
#define ATSC3LLSSLTServiceNodeAttribute_globalServiceID                 0x0002
#define ATSC3LLSSLTServiceNodeAttribute_sltSvcSeqNum                    0x0004
#define ATSC3LLSSLTServiceNodeAttribute_protected                       0x0008
#define ATSC3LLSSLTServiceNodeAttribute_majorChannelNo                  0x0010
#define ATSC3LLSSLTServiceNodeAttribute_minorChannelNo                  0x0020
#define ATSC3LLSSLTServiceNodeAttribute_serviceCategory                 0x0040
#define ATSC3LLSSLTServiceNodeAttribute_shortServiceName                0x0080
#define ATSC3LLSSLTServiceNodeAttribute_hidden                          0x0100
#define ATSC3LLSSLTServiceNodeAttribute_essential                       0x0200
#define ATSC3LLSSLTServiceNodeAttribute_broadbandAccessRequired         0x0400
#define ATSC3LLSSLTServiceNodeAttribute_svcCapabilities                 0x0800
#define ATSC3LLSSLTServiceNodeChildNode_BroadcastSvcSignaling           0x1000
#define ATSC3LLSSLTServiceNodeChildNode_svcInetUrl                      0x2000
#define ATSC3LLSSLTServiceNodeChildNode_otherBsid                       0x4000

// See A/331 6.2
typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t bsid;              // Identifies the one or more Broadcast Streams 
                                // comprising the Services
    int32_t sltCapabilitiesDataOffset;  // Reqired capabilities for decoding and
                                // meaningfully presenting the content for all
                                // the services in this SLT instance.
    ATSC3LLSSLTInetUrlNode sltInetUrl;  // Base URL to acquire ESG or service 
                                // layer signaling files available via 
                                // broadband for services in this SLT.

    EnXMLParserDataOffsetArray services; // Array of
                                // ATSC3LLSSLTServiceNode

    EnXMLParserDataBuffer dataBuffer;
} ATSC3LLSSLT_SLTNode;

#define ATSC3LLSSLTNodeAttribute_XMLNS                          0x0001
#define ATSC3LLSSLTNodeAttribute_BSID                           0x0002
#define ATSC3LLSSLTNodeAttribute_SLTCapabilities                0x0004
#define ATSC3LLSSLTNodeChildNode_SLTInetUrl                     0x0008
#define ATSC3LLSSLTNodeChildNode_Service                        0x0010

typedef enum
{
    ATSC3LLSSLTNodeType_Invalid = 0,
    ATSC3LLSSLTNodeType_SLT = 1,
    ATSC3LLSSLTNodeType_Service = 2,
    ATSC3LLSSLTNodeType_InetUrl = 3,
    ATSC3LLSSLTNodeType_OtherBsid = 4,
    ATSC3LLSSLTNodeType_BroadcastSvcSignaling = 5,
} ATSC3LLSSLTNodeType;

ENStatus
ATSC3LLSParseSLT(const char *pSLT, int32_t sSLT, 
    ATSC3LLSSLT_SLTNode *pParsedSLT);

ENStatus
ATSC3LLSFreeParsedSLT(ATSC3LLSSLT_SLTNode *pParsedSLT);

#endif
