/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 * 
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3ROUTESESSION_H
#define ENATSC3ROUTESESSION_H

typedef struct _ENATSC3ROUTESession *ENATSC3ROUTESessionHandle_t;
typedef struct _ENATSC3EROUTESLSSTSIDNode *ENATSC3EROUTESLSSTSIDNodePtr;

// ENATSC3ROUTESessionType identifies the type of objects delivered by the
// ROUTE session, for example DASH or ATSC 3 ESG
typedef enum
{
    ENATSC3ROUTESessionType_Invalid = 0,
    ENATSC3ROUTESessionType_ATSC3_DASH = 1,
    ENATSC3ROUTESessionType_ATSC3_ESG = 2,
} ENATSC3ROUTESessionType;

// ENATSC3ROUTESessionIdType indicates where pId was derived from - either the
// representation id (as specified in the STSID for the LCT channel), or
// the URL of the first segment (as derived from the segment template).
typedef enum
{
    ENATSC3ROUTESessionIdType_Unknown = 0,
    ENATSC3ROUTESessionIdType_RepId = 1,
    ENATSC3ROUTESessionIdType_Segment0URL = 2,
} ENATSC3ROUTESessionIdType;

// The MPD, HELD, and STSID callbacks return ENStatus_Success if the
// SLS was properly handled. If all registered callbacks for a document type
// return Success then the document is consdidered accepted and no further
// callbacks will occur for that document type until the version or crc32
// changes. If any callback for a document type fails the object then the
// document is considered not accepted and the next instance of the document
// received will result in callbacks. If a callback accepted a document it 
// must be prepared to receive a document with the same version and crc32
// (and most likely ignore it).

typedef ENStatus (*ENATSC3ROUTESessionMPDAvailableCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, const char *pMPD, 
    int32_t sMPD, int32_t version, uint32_t crc32, void *pUserData);

typedef ENStatus (*ENATSC3ROUTESessionHELDAvailableCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, const char *pHELD, 
    int32_t sHELD, int32_t version, uint32_t crc32, void *pUserData);

typedef ENStatus (*ENATSC3ROUTESessionSTSIDAvailableCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, const char *pSTSID, 
    int32_t sSTSID, ENATSC3ROUTESLSSTSIDNodePtr pParsedSTSID, int32_t version, 
    uint32_t crc32, void *pUserData);

typedef void (*ENATSC3ROUTESessionObjectDataCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, uint32_t tsi, 
    uint32_t toi, uint8_t codepoint, int32_t formatId, int32_t frag, 
    int32_t order, const char *pId, ENATSC3ROUTESessionIdType idType, 
    const uint8_t *pPayload, int32_t payloadLength, int64_t objectPosition, 
    int64_t tolTransferLength, int64_t ftiTransferLength, int32_t fError, 
    void *pUserData);
 
typedef void (*ENATSC3ROUTESessionSessionResetCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, void *pUserData);

typedef void (*ENATSC3ROUTESessionLCTChannelAddedCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, uint32_t addr, 
    uint16_t port, uint32_t tsi, const char *pId, 
    ENATSC3ROUTESessionIdType idType, void *pUserData);

typedef void (*ENATSC3ROUTESessionLCTChannelRemovedCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, uint32_t addr, 
    uint16_t port, uint32_t tsi, void *pUserData);

// Note that the SLS available callback is informative and clients cannot
// reject the SLS, as they can with the S-TSID, MPD, or HELD
typedef void (*ENATSC3ROUTESessionSLSAvailableCallbackFunction)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, const char *pURL, 
    const uint8_t *pSLS, int32_t sSLS, int32_t fGzipped, void *pUserData);

typedef void (*ENATSC3ROUTESessionIPAddressAddCallbackFunction)(uint32_t addr, 
    uint16_t port, void *pUserData);

typedef void (*ENATSC3ROUTESessionIPAddressRemoveCallbackFunction)(
    uint32_t addr, uint16_t port, void *pUserData);

typedef void (*ENATSC3ROUTESessionIPAddressCommitCallbackFunction)(
    void *pUserData);

ENStatus ENATSC3ROUTESessionAllocate(
    ENATSC3ROUTESessionType routeSessionType, uint32_t slsAddr, int32_t slsPort,
    ENATSC3ROUTESessionIPAddressAddCallbackFunction IPAddressAddCallback, 
    ENATSC3ROUTESessionIPAddressRemoveCallbackFunction IPAddressRemoveCallback, 
    ENATSC3ROUTESessionIPAddressCommitCallbackFunction IPAddressCommitCallback, 
    void *pIPAddrCallbackUserData, char *pDebugName, 
    ENATSC3ROUTESessionHandle_t *phROUTESession);

ENStatus ENATSC3ROUTESessionFree(
    ENATSC3ROUTESessionHandle_t hROUTESession);

void ENATSC3ROUTESessionSetBitrateStatisticsEnabled(
    ENATSC3ROUTESessionHandle_t hROUTESession, int32_t fEnabled);

ENStatus ENATSC3ROUTESessionSetVerbosity
    (ENATSC3ROUTESessionHandle_t hROUTESession, ENVerbosity verbosity);

ENStatus ENATSC3ROUTESessionReset(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    struct timespec *pPacketTimeUTC, uint64_t packetNumber);

ENStatus ENATSC3ROUTESessionRegisterMPDAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionMPDAvailableCallbackFunction MPDAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionUnregisterMPDAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionMPDAvailableCallbackFunction MPDAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionRegisterHELDAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionHELDAvailableCallbackFunction HELDAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionUnregisterHELDAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionHELDAvailableCallbackFunction HELDAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionRegisterSTSIDAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionSTSIDAvailableCallbackFunction STSIDAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionUnregisterSTSIDAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionSTSIDAvailableCallbackFunction STSIDAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionRegisterSLSAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionSLSAvailableCallbackFunction SLSAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionUnregisterSLSAvailableCallback(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionSLSAvailableCallbackFunction SLSAvailableCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionRegisterLCTChannelCallbacks(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionLCTChannelAddedCallbackFunction LCTChannelAddedCallback,
    ENATSC3ROUTESessionLCTChannelRemovedCallbackFunction LCTChannelRemovedCallback,
    void *pUserData);

ENStatus ENATSC3ROUTESessionUnregisterLCTChannelCallbacks(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionLCTChannelAddedCallbackFunction LCTChannelAddedCallback,
    ENATSC3ROUTESessionLCTChannelRemovedCallbackFunction LCTChannelRemovedCallback,
    void *pUserData);

ENStatus ENATSC3ROUTESessionRegisterObjectCallbacks(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionObjectDataCallbackFunction ObjectDataCallback, 
    ENATSC3ROUTESessionSessionResetCallbackFunction SessionResetCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionUnregisterObjectCallbacks(
    ENATSC3ROUTESessionHandle_t hROUTESession,
    ENATSC3ROUTESessionObjectDataCallbackFunction ObjectDataCallback, 
    ENATSC3ROUTESessionSessionResetCallbackFunction SessionResetCallback, 
    void *pUserData);

ENStatus ENATSC3ROUTESessionGetObjectURLAndContentType(
    ENATSC3ROUTESessionHandle_t hROUTESession, uint32_t tsi, uint32_t toi, 
    char *pURL, int32_t sURL, char *pContentType, int32_t sContentType);

ENStatus ENATSC3ROUTESessionGetEntityModeObjectURLAndContentType(
    ENATSC3ROUTESessionHandle_t hROUTESession, uint32_t tsi,
    const uint8_t *pPayload, int32_t payloadLength, int32_t *pPayloadOffset,
    char *pURL, int32_t sURL, char *pContentType, int32_t sContentType);

ENStatus ENATSC3ROUTESessionProcessUDPPacket(
    ENATSC3ROUTESessionHandle_t hROUTESession, struct timespec *pPacketTimeUTC, 
    uint64_t packetNumber, uint8_t plpId, const uint8_t *pPayload, 
    int32_t payloadLength);

ENStatus ENATSC3ROUTESessionSetShadowFilePath(
    ENATSC3ROUTESessionHandle_t hROUTESession, char *pPath); 

#endif
