/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENXML_H
#define ENXML_H

typedef char IPV4Address[16];

typedef struct 
{
    int32_t num;
    int32_t den;
} EnXMLFraction;

typedef  struct
{
    int32_t w;
    int32_t h;
} EnXMLRatio;

typedef struct
{
    uint32_t nodeType;
    uint64_t presentFlags;
    uint8_t textPresent;
} EnXMLParserNodeBase;

typedef struct _EnXMLParserAttribute
{
    int32_t id;
    int32_t sName;
    char *pName;
    ENStatus (*handler)(struct _EnXMLParserAttribute *pAttribute,
        const xmlChar *pValue, int32_t sValue, xmlTextReaderPtr reader,
        void *pUserData, EnXMLParserNodeBase *pParentNode);
} EnXMLParserAttribute;

typedef struct _EnXMLParserChildNode
{
    int32_t id;
    int32_t sName;
    char *pName;
    ENStatus (*handler)(struct _EnXMLParserChildNode *pChildNode,
        xmlTextReaderPtr reader, void *pUserData,
        EnXMLParserNodeBase *pParentNode);
} EnXMLParserChildNode;

typedef struct _EnXMLParserNode
{
    int32_t id;
    char *pName;
    EnXMLParserAttribute *pAttributes;
    int32_t nAttributes;
    EnXMLParserChildNode *pChildNodes;
    int32_t nChildNodes;
    ENStatus (*textHandler)(struct _EnXMLParserNode *pNodeData,
        const xmlChar *pValue, int32_t sValue, xmlTextReaderPtr reader,
        void *pUserData, EnXMLParserNodeBase *pParentNode);
} EnXMLParserNode;

#define EnXMLParserDataBufferGetPointer(pDataBuffer, offset) ((void *)((pDataBuffer)->pBuffer + offset))
typedef struct _EnXMLParserDataBuffer
{
    uint8_t *pBuffer;
    int32_t sBuffer;
    int32_t nBufferBytesUsed;
    int32_t increment;
} EnXMLParserDataBuffer;

#define EnXMLParserDataOffsetArrayGetPointer(pDataOffsetArray, index) ((void *)((pDataOffsetArray)->pDataBuffer->pBuffer + ((int32_t *)((pDataOffsetArray)->pDataBuffer->pBuffer + (pDataOffsetArray)->dataBufferOffset))[index]))
typedef struct _EnXMLParserDataOffsetArray
{
    // The data buffer from which array index space is allocated.
    EnXMLParserDataBuffer *pDataBuffer;
    int32_t sInitialSize;

    // The offset in DataBuffer where the array resides
    int32_t dataBufferOffset;
    int32_t nOffsetsAllocated;
    int32_t nOffsetsUsed;
    int32_t increment;
} EnXMLParserDataOffsetArray;

ENStatus EnXMLParserDataOffsetArrayInitialize(
    EnXMLParserDataOffsetArray *pDataOffsetArray, 
    EnXMLParserDataBuffer *pDataBuffer, int32_t sInitial, int32_t sIncrement);
ENStatus EnXMLParserDataOffsetArrayAppendOffset(
    EnXMLParserDataOffsetArray *pDataOffsetArray, int32_t offset);

ENStatus EnXMLParserDataBufferInitialize(EnXMLParserDataBuffer *pDataBuffer,
    int32_t sInitial, int32_t sIncrement);
ENStatus EnXMLParserDataBufferFree(EnXMLParserDataBuffer *pDataBuffer);
ENStatus EnXMLParserDataBufferAddString(EnXMLParserDataBuffer *pDataBuffer, 
    const char *pString, int32_t sString, int32_t *pDataBufferOffset);
ENStatus EnXMLParserDataBufferAddStringAndZeroTerminate(
    EnXMLParserDataBuffer *pDataBuffer, const char *pString, 
    int32_t sString, int32_t *pDataBufferOffset);
ENStatus EnXMLParserDataBufferAllocateSpace(EnXMLParserDataBuffer *pDataBuffer, 
    int32_t nBytes, int32_t *pDataBufferOffset);

ENStatus EnXMLParser_antoll(const char *num, int32_t n, int64_t *pValue);
ENStatus EnXMLParser_antol(const char *num, int32_t n, int32_t *pValue);
ENStatus EnXMLParser_antoull(const char *num, int32_t n, uint64_t *pValue);
ENStatus EnXMLParser_antoul(const char *num, int32_t n, uint32_t *pValue);
ENStatus EnXMLParser_antof(const char *num, int32_t n, double *pValue);

ENStatus EnXMLParserStringToFraction(const char *pValue, int32_t sValue, 
    EnXMLFraction *pFraction);
ENStatus EnXMLParserStringToRatio(const char *pValue, int32_t sValue, 
    EnXMLRatio *pRatio);
ENStatus EnXMLParserStringToBool(const char *pValue, int32_t sValue, 
    int32_t *pBoolVal);
ENStatus EnXMLParserXMLDurationToSeconds(const char *pTime, int32_t sTime,
    int64_t *pSeconds, int64_t *pMilliseconds);
ENStatus EnXMLParserDateToDaysSinceEpoch(int32_t year, int32_t month, 
    int32_t day, int64_t *pDays);
ENStatus EnXMLParserZTimeToSecondsUTC(const char *pTime, int32_t sTime, 
    int64_t *pSecondsUTC, int64_t *pMilliseconds);
ENStatus EnXMLParserDateTimeToSecondsUTC(const char *pDateTime, 
    int32_t sDateTime, int64_t *pSecondsUTC, int64_t *pMilliseconds);
ENStatus EnXMLParserNTPSecondsToSecondsUTC(int64_t ntpSeconds, 
    int64_t *pSecondsUTC);
ENStatus EnXMLParserDayTimeDurationToSeconds(const char *pDuration,
    int32_t sDuration, int64_t *pSeconds, int64_t *pMilliseconds);

ENStatus EnXMLParserProcessNode(EnXMLParserNode *pNodeData, 
    xmlTextReaderPtr reader, void *pUserData, 
    EnXMLParserNodeBase *pParentNode);

#endif
